-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;					

entity sha2_pad is
    generic(w:integer:=32 );
    port 	( 	din : in  STD_LOGIC_VECTOR (w-1 downto 0);
           		dout : out  STD_LOGIC_VECTOR (w-1 downto 0);
			  	sel_pad : in STD_LOGIC_VECTOR (1 downto 0);
			  	sel_pad_location : in STD_LOGIC_VECTOR (w/8-1 downto 0);	
				sel_din : in std_logic_vector (w/8-1 downto 0);
			  	clk  : in   STD_LOGIC;
			  	rst : in std_logic;
		     	en_size : in std_logic);
end sha2_pad;

architecture struct of sha2_pad is		  
	signal dout_wire_byte : STD_LOGIC_VECTOR (w-1 downto 0);
	signal msg_size_wire  : STD_LOGIC_VECTOR (2*w-1 downto 0);
	signal din_r : STD_LOGIC_VECTOR (w-1 downto 0);
begin
	din_r <= '0' & din(w-2 downto 0) ;			
	
	with sel_pad select
	dout <= msg_size_wire(2*w-1 downto w) when "00",
			msg_size_wire(w-1 downto 0) when "01",
			dout_wire_byte when others;		 

	byte_1:entity work.byte_pad(struct)
		generic map ( w => w)	
		port map (din => din , dout => dout_wire_byte , sel_pad_location => sel_pad_location, sel_din => sel_din );
	
	msglen_2:entity work.msg_len(struct)
		generic map ( isize => w, osize => 2*w, adder_size => 16)	
	    port map(din => din_r,msg_size=>msg_size_wire ,clk=> clk, rst=> rst, en => en_size);	
end struct;
